import { InjectionKey } from 'vue'
import { createStore, Store, createLogger } from 'vuex'
import { LOGIN_SUCCESS, LOGOUT, TITLE } from './mutation-types'
import { FETCH_ACCOUNT, FETCH_BUDGET, GET, REGISTER, IMPORT_YNAB, LOGIN, NEW_BUDGET, POST, SET_CURRENT_ACCOUNT, SET_CURRENT_BUDGET } from './action-types'
import { budgetStore } from './budget'

export interface State {
    Session: {
        Token?: string
        User?: string
    },
    ShowMenu?: boolean,
    ExpandMenu?: boolean,
    Budgets: Map<string, Budget>,
    CurrentBudgetID?: string,
}

export interface Budget {
    ID: string
    Name: string
    AvailableBalance: number
}

export const key: InjectionKey<Store<State>> = Symbol()

export const store = createStore<State>({
    state: {
        Session: {
            Token: undefined,
            User: undefined
        },
        ShowMenu: undefined,
        Budgets: new Map<string, Budget>(),
        CurrentBudgetID: undefined,
    },
    mutations: {
        deleteBudget(state: State, budgetid: string) {
            state.Budgets.delete(budgetid)
        },
        toggleMenu(state) {
            state.ShowMenu = !state.ShowMenu;
        },
        toggleMenuSize(state) {
            state.ExpandMenu = !state.ExpandMenu;
        },
        initializeStore(state) {
            const store = localStorage.getItem("store");
            if (!store)
                return;

            const restoredState = JSON.parse(store);
            if (!restoredState)
                return;

            state.Session = restoredState.Session;
            state.CurrentBudgetID = restoredState.CurrentBudgetID;
            state.ShowMenu = restoredState.ShowMenu;
            state.ExpandMenu = restoredState.ExpandMenu;

            for (const budget of restoredState.Budgets || []) {
                state.Budgets.set(budget[0], budget[1]);
            }
        },
        [TITLE](state, title) {
            document.title = "Budgeteer - " + title;
        },
        [LOGIN_SUCCESS](state, result) {
            state.Session = {
                User: result.User,
                Token: result.Token
            };
            for (const budget of result.Budgets) {
                state.Budgets.set(budget.ID, budget)
            }
        },
        addBudget(state, budget) {
            state.Budgets.set(budget.ID, budget);
        },
        [LOGOUT](state) {
            state.Session = { Token: undefined, User: undefined };
            state.Budgets.clear();
        },
        setCurrentBudgetID(state, budgetid) {
            state.CurrentBudgetID = budgetid;
        },
    },
    actions: {
        [LOGIN]({ state, commit }, login) {
            return fetch("/api/v1/user/login", { method: "POST", body: JSON.stringify(login) })
                .then(x => x.json())
                .then(x => commit(LOGIN_SUCCESS, x))
        },
        [REGISTER]({ state, commit }, login) {
            return fetch("/api/v1/user/register", { method: "POST", body: JSON.stringify(login) })
                .then(x => x.json())
                .then(x => commit(LOGIN_SUCCESS, x))
        },
        [IMPORT_YNAB]({ getters, dispatch }, formData) {
            return dispatch("POST", { path: "/budget/" + getters.CurrentBudget.ID + "/import/ynab", body: formData });
        },
        [GET]({ getters }, { path }) {
            return fetch("/api/v1" + path, {
                headers: getters.AuthHeaders,
            })
        },
        [POST]({ getters }, { path, body }) {
            return fetch("/api/v1" + path, {
                method: "POST",
                headers: getters.AuthHeaders,
                body: body,
            })
        },
        /*async fetchDashboard ({state, commit, rootState}) {
            const response = await fetch("/api/v1/dashboard", {
                headers: {
                    'Authorization': 'Bearer ' + rootState.Session.Token
                }
            })
            const data = await response.json();
            commit("setBudgets", data.Budgets);
        },*/
        async [NEW_BUDGET]({ state, commit, dispatch, rootState }, budgetName) {
            const result = await dispatch("POST", {
                path: "/budget/new",
                body: JSON.stringify({ name: budgetName })
            });
            const response = await result.json();
            commit("addBudget", response)
        },
        async [SET_CURRENT_BUDGET]({ state, commit, dispatch, rootState }, budgetid) {
            commit("setCurrentBudgetID", budgetid);

            if (budgetid == null)
                return

            await dispatch(FETCH_BUDGET, budgetid)
        },
    },
    getters: {
        Budgets(state) {
            return state.Budgets.values();
        },
        AuthHeaders(state) {
            return {
                'Authorization': 'Bearer ' + state.Session.Token
            }
        },
        CurrentBudget(state) : Budget | undefined {
            if (state.CurrentBudgetID == null)
                return undefined;

            return state.Budgets.get(state.CurrentBudgetID);
        },
        CurrentBudgetID(state) : string | undefined {
            return state.CurrentBudgetID;
        },
        CurrentBudgetName(state) : string {
            if (state.CurrentBudgetID == null)
                return "";

            const currentBudget = state.Budgets.get(state.CurrentBudgetID);
            if(currentBudget != undefined)
                return currentBudget.Name;
            return "";
        },
    },
    plugins: [createLogger()],
    modules: {
        budget: budgetStore
    }
})

store.subscribe((mutation, state) => {
    let persistedState = {
        Session: state.Session,
        Budgets: [...state.Budgets],
//        Accounts: [...state.Accounts],
        CurrentBudgetID: state.CurrentBudgetID,
        //CurrentAccountID: state.CurrentAccountID,
        ExpandMenu: state.ExpandMenu,
        ShowMenu: state.ShowMenu
    }
    localStorage.setItem("store", JSON.stringify(persistedState));
})